@@ -12,7 +12,7 @@ Lingua::EN::Inflect::Phrase - Inflect short English Phrases
 
 =cut
 
-our $VERSION = '0.04';
+our $VERSION = '0.05';
 
 =head1 SYNOPSIS
 
@@ -46,7 +46,9 @@ our @EXPORT_OK = qw/to_PL to_S/;
 
 =cut
 
-my $NOUN = qr{(\S+)/(NN|CD)};
+my $NOUN         = qr{(\S+)/NNS?\b};
+my $MAYBE_NOUN   = qr{(\S+)/(?:NNS?|CD|JJ)\b};
+my $NOUN_OR_VERB = qr{(\S+)/(?:NNS?|CD|JJ|VB[A-Z]?)\b};
 
 my $tagger;
 
@@ -65,19 +67,58 @@ sub _inflect {
   my ($phrase, $want_plural, $method) = @_;
   my $want_singular = not $want_plural;
 
+# Do not tag initial number, if any, unless it's "1" which is handled
+# separately. Regex is from perldoc -q 'is a number'.
+  my ($number, $padding, $rest) =
+    $phrase =~ /^(\s*(?:[+-]?)(?=\d|\.\d)\d*(?:\.\d*)?(?:[Ee](?:[+-]?\d+))?)(\s*)(.*)$/;
+
+  my $tagged;
   $tagger ||= Lingua::EN::Tagger->new;
 
-  my $tagged = $tagger->get_readable($phrase);
-  my $noun;
+  if ($number && $number != 1) {
+    $tagged = $number . $padding . $tagger->get_readable($rest);
+  }
+  else {
+    $tagged = $tagger->get_readable($phrase);
+  }
 
-# last noun before a preposition/conjunction
-# or last noun
+  my $force_singular = $tagged =~ m{
+    ^ \s* (?:(?:a|the)/DET)?
+    \s* (?:1|one|single)/(?:JJ|NN|CD)\b
+    (?!\s* (?:\S+/CC\b | [0-9/]+/CD\b))
+  }x;
+
+  my ($noun, $tag);
+
+# last noun (or verb that could be a noun) before a preposition/conjunction
+# or last noun/verb
   if ((($noun) = $tagged =~ m{$NOUN (?!.*/(?:NN|CD|JJ).*/(?:CC|IN)) .* /(?:CC|IN)}x) or
-      (($noun) = $tagged =~ m{$NOUN (?!.*/(?:NN|CD|JJ))}x)) {
-    my $is_plural = Lingua::EN::Inflect::Number::number($noun) ne 's';
-    my $inflected_noun = _inflect_noun($noun, $is_plural, $want_plural, $method);
+      (($noun) = $tagged =~ m{$NOUN (?!.*/(?:NN|CD|JJ))}x) or
+      (($noun) = $tagged =~ m{$MAYBE_NOUN (?!.*/(?:NN|CD|JJ).*/(?:CC|IN)) .* /(?:CC|IN)}x) or
+      (($noun) = $tagged =~ m{$MAYBE_NOUN (?!.*/(?:NN|CD|JJ))}x) or
+      (($noun) = $tagged =~ m{$NOUN_OR_VERB (?!.*/(?:NN|CD|JJ|VB[A-Z]?).*/(?:CC|IN)) .* /(?:CC|IN)}x) or
+      (($noun) = $tagged =~ m{$NOUN_OR_VERB (?!.*/(?:NN|CD|JJ|VB[A-Z]?))}x)) {
+    my @pos = ($-[1], $+[1]);
+    my $inflected_noun;
 
-    substr($tagged, $-[1], ($+[1] - $-[1])) = $inflected_noun if $inflected_noun;
+# special case phrases like "2 right braces"
+    if (lc($noun) eq 'right') {
+      (($noun) = $tagged =~ m{$NOUN_OR_VERB (?!.*/(?:NN|CD|JJ|VB[A-Z]?).*/(?:CC|IN)) .* /(?:CC|IN)}x) or
+      (($noun) = $tagged =~ m{$NOUN_OR_VERB (?!.*/(?:NN|CD|JJ|VB[A-Z]?))}x);
+
+      @pos = ($-[1], $+[1]);
+    }
+
+    if ($force_singular) {
+      $inflected_noun = Lingua::EN::Inflect::Number::to_S($noun);
+    }
+    else {
+      my $is_plural = Lingua::EN::Inflect::Number::number($noun) ne 's';
+
+      $inflected_noun = _inflect_noun($noun, $is_plural, $want_plural, $method);
+    }
+
+    substr($tagged, $pos[0], ($pos[1] - $pos[0])) = $inflected_noun if $inflected_noun;
 
     ($phrase = $tagged) =~ s{/[A-Z]+}{}g;
   }
@@ -85,7 +126,10 @@ sub _inflect {
   else {
     my $number = Lingua::EN::Inflect::Number::number($phrase);
 
-    if (($want_plural && $number ne 'p') || ($want_singular && $number ne 's')) {
+    if ($force_singular) {
+      $phrase = Lingua::EN::Inflect::Number::to_S($phrase);
+    }
+    elsif (($want_plural && $number ne 'p') || ($want_singular && $number ne 's')) {
       $phrase = $phrase->$method;
     }
   }
